c ------------------------------------------------------------------------
c
c  ** NOTE - DETECTORS WITH RESPONSES THAT DON'T GO BELOW A keV SHOULD BE
c     EXTENDED WITH THE 'ENERGIES' COMMAND 
c
c   FOR EXAMPLE, WITH RXTE:
c   >    energies 0.2 50 498 lin 
c   
c
c   This routine is designd to give a physically motivated
c   prescription to build a powerlaw model using a seed spectrum.
c   Photon number is preserved in the output spectrum.  
c
c     USER INPUT:
c
c     param(1) - Normalization of the spectrum. 
c     param(2) - Spectral index 
c     param(3) - Flag - toggles between downscattering+upscattering (0)
c                       and upscattering only (1)
c------------------------------------------------------------------------
c     XSPEC INPUT
c     
c     2 Vectors:
c     -----------------
c     1) Energy
c     2) Flux (Energy)
c
c
c     EAR(0:NE) - array of observed energy bins in keV
c     NE        - # observed energy bins
c     PARAM     - list of parameters
c     IFL - spectrum # irrelevant parameter? -  check in kerrftab.f 
c           and XSPEC11 /12 manual [Appendix C]
c     PHOTAR() - an array of photon flux in each bin of energy PHOTAR(NE)
c     PHOTER() - formal output for errors
c
c
c
c
c  EXAMPLE:
c      model phabs(simpl(diskbb))
c     DONTHCOMP(ear, ne, pars, spectrumNumber, photar, apPrim.get());
c==========================================================================

      subroutine simplcutx(ear,ne,param,IFL,photar,photer) 

      integer i,j
      double precision fsc,refl

      double precision ecum, ecumi
      real ear(0:ne),param(4),photar(ne),photer(ne),kte,ecut,gamma
      real tmparr(ne),enavgam(ne),tmpint(ne),nthprim(ne),nthpar(2)

      real tmpcum1(ne),tmpcumi(ne),tmparrt(ne),earav(ne),switch

c--------------------------
      INCLUDE 'xspec.inc'
c--------------------------

      
      fsc=param(2)
      gamma=param(1)
      refl=param(3)
      kte=param(4)

      
      nthpar(1) = gamma
      nthpar(2) = kte

      switch=0.
      
      
c     WORKAROUND to avoid pole at gamma=1
      IF (gamma .eq. 1) then 
         gamma=1.001
      ENDIF

      
c     Initialize arrays
      
      do 10 i=0,ne
         if (i.ne.0) then
            tmparr(i)=0.
            
            enavgam(i) =(0.5*(ear(i-1)+ear(i)))**(-gamma)
            earav(i)   =(0.5*(ear(i-1)+ear(i)))
            
            tmparrt(i) = 0.
         endif
 10   continue
      





      IF (kte .lt. 0) THEN      !ECUT IMPLEMENTATION
         ecut = -kte
         
         IF (switch .ge. 1) THEN !UPSCATTERING ONLY!
C     UPSCATTERING ONLY FROM A POWERLAW ELECTRON DISTRIBUTION 
c     Loop over energy bins


c     FOR EACH STARTING BIN, GET THE FULL SPECTRUM, AND SUM IT
            
            do 25 i=1,ne
               ecum=0.
               ecumi=0.
               do 20 j=1,ne  
                  if (j.lt.i) then 
                     tmpint(j)=0.
                     tmparr(j)=0.
               
                     
                     tmpcum1(j)=ecum
                     tmpcumi(j)=ecumi
               
                  else if (j.ge.i) then
                     tmpint(j)=enavgam(j)/enavgam(i)*exp(-earav(j)/ecut)
                     tmparr(j)=photar(i)*fsc*tmpint(j)
                     
                     ecum = ecum +tmparr(j)*(ear(j)-ear(j-1))
                     ecumi= ecumi+tmpint(j)*(ear(j)-ear(j-1))
                     
                     tmpcum1(j)=ecum
                     tmpcumi(j)=ecumi
                     
                     if (j.eq.ne) then
                        tmparr=tmparr/ecumi
                        tmparrt = tmparrt+tmparr
                     endif
                  endif
 20            continue
 25         continue

            
         ELSE IF (switch.lt.1) then 
         
         
            do 45 i=1,ne
               ecum=0.
               ecumi=0.
               
               do 40 j=1,ne  
                  if (j.lt.i) then 
                     tmpint(j)=earav(j)**(gamma+1.)
                     tmparr(j)=photar(i)*fsc*tmpint(j)
                     
                     ecum = ecum +tmparr(j)*(ear(j)-ear(j-1))
                     ecumi= ecumi+tmpint(j)*(ear(j)-ear(j-1))
                     
                     tmpcum1(j)=ecum
                     tmpcumi(j)=ecumi
                  
                  else if (j.ge.i) then
                     tmpint(j)=enavgam(j)/enavgam(i)*exp(-earav(j)/ecut)
     $                    *earav(i)**(gamma+1.)
                     tmparr(j)=photar(i)*fsc*tmpint(j)
                  
                     ecum = ecum +tmparr(j)*(ear(j)-ear(j-1))
                     ecumi= ecumi+tmpint(j)*(ear(j)-ear(j-1))
                     
                     tmpcum1(j)=ecum
                     tmpcumi(j)=ecumi
                  
                     if (j.eq.ne) then
                        tmparr=tmparr/ecumi
                        tmparrt = tmparrt+tmparr
                     endif
                  endif
               

 40            continue
 45         continue

         ENDIF

 
         do 50 i=1,ne
            tmparrt(i)=tmparrt(i)*(ear(i)-ear(i-1))
 50      continue
      

      ELSE                 ! NTHCOMP CALL


         do i=1,ne
            tmparrt(i) =photar(i)/(ear(i)-ear(i-1))  ! initialize  TO PHOTON DENSITY!
         end do


         call   runthcomp(ear,ne,nthpar,IFL,tmparrt,nthprim)

         
         
         psum = 0.
         tsum = 0.
         do 55 j=1,ne   ! SUM THE PHOTONS  (*** INDEX RUN MAY BE TRUNCATED TO AVOID BAD LAST CHAN ****)  
            psum = psum+photar(j)
            tsum = tsum+tmparrt(j)
 55      continue

        tmparrt = tmparrt/tsum*psum*fsc  ! normalize the output
 
      ENDIF
      


C       PHOTAR UNITS ARE:
C      (Photons/cm^2/s per BIN) (i.e, flx integrated over the bin width)

      if (refl .ge. 0.0) then 
         photar=1./(refl+1.)*tmparrt+(1.-fsc)*photar
      else
         photar=1./abs(refl-1)*tmparrt
      endif
      
      
      RETURN
      
      END





C ----------------------------------------------------
C            Excerpted / Edited from (do)NTHCOMP
      
      SUBROUTINE runthcomp(Ear,Ne,Param,Ifl,spec,prim)
      implicit none
      INTEGER Ne , Ifl
      REAL Param(*),Ear(0:Ne),spec(Ne), prim(0:Ne),seedspec(Ne),xear(Ne)
c
c     driver for the Comptonization code solving Kompaneets equation
c     seed photons - (disc) blackbody
c     reflection + Fe line with smearing
c     
c     Version optimized for a number of data files but with the same values 
c     of parameters:
c        computes the model when ifl=1, in broad energy range, and rebins
c        it only for subsequent files
c
c     number of model parameters: 16
c     1: photon spectral index 
c     2: plasma temperature in keV 
c CC    3: (disc)blackbody temperature in keV   
c CC    4: type of seed spectrum (0-blackbody, 1-diskbb)
c CC    5: redshift

c      INCLUDE 'xspec.inc'

      INTEGER n , ierr, np
c     SPP is external (xansrc/functions/xsnteea.f)
      REAL xn , normfac , normlum, SPP, pa0(2)

      integer nth
      REAL xth(0:Ne) , spt(0:Ne), xprim(0:Ne)

      LOGICAL fl0, IFLAG

      real    z_red
      integer i,j, jl


      seedspec = spec
      
c     xtot is the energy array (units m_e c^2)
c     spnth is the nonthermal spectrum alone (E F_E)
c     sptot is the total spectrum array (E F_E), = spref if no reflection
      
      ierr = 0
      
      z_red = 0. !param(5)
         
c  calculate internal source spectrum if necessary
c  (only for the first file)

         np = 2
         xear = ear/511.

         call jsCompton(seedspec,param(2)/511.,param(1),
     >        xear,xth,Ne,spt,IFLAG)
         
         normfac = 1.
         
      if (IFLAG) then 
         j=1
         do i=0,Ne
            do while (j .le. Ne .and. 511.*xth(j) .lt. ear(i)*(1+z_red))
               j=j+1
            end do
            if (j .lt. Ne) then
               if (j .gt. 1) then
                  jl = j-1
                  xprim(i) = spt(jl)+(ear(i)/511.*(1.0+z_red)-xth(jl))
     >                 *(spt(jl+1)-spt(jl))/(xth(jl+1)-xth(jl))
               else
                  xprim(i)=spt(1)
               endif
            else
            xprim(i)=spt(j)
            endif
          end do
      else
         xprim=spt
      endif

      xprim(Ne) = spt(Ne)

      do i=1,Ne
         spec(i) = 0.5*(xprim(i)/ear(i)**2+xprim(i-1)/ear(i-1)**2)
     >        *(ear(i)-ear(i-1))*normfac
      end do
            

      RETURN
      END
      


      subroutine jsCompton(specin,theta,gamma,xin,x,nen,sptot,IFLAG) 
CC  -- JS - x should be per 511
c     version: January 96
c
c
c     Thermal Comptonization; solves Kompaneets eq. with some
c     relativistic corrections. See Lightman & Zdziarski (1987), ApJ
c     The seed spectrum is blackbody.
      IMPLICIT NONE
      integer nen
      real xmin,xmax,xnr,xr,taukn,arg,flz,planck,pi
      real sptot(0:nen),x(0:nen),dphesc(nen),dphdot(nen)
      real delta,deltal,xin(0:nen)
     >  ,rel(nen),bet(nen),c2(nen),specin(nen)
      integer j,jnr,jrel, tmp1(1), jl, i
      DOUBLE PRECISION w,w1,z1,z2,z3,z4,z5,z6
      real tautom, dif1
c  input parameters:
      real tempbb,theta,gamma
      logical IFLAG
      
c use internally Thomson optical depth
      tautom=sqrt(2.25+3/(theta*((gamma+.5)**2-2.25)))-1.5
c
      pi=3.14159
      sptot(0)=0.
c clear arrays (important for repeated calls)
      do 31 j=1,nen
        dphesc(j)=0
        dphdot(j)=0
        rel(j)=0
        bet(j)=0
        c2(j)=0
        sptot(j)=0.
  31  continue
C
C JMAX - # OF PHOTON ENERGIES
C  
c delta is the 10-log interval of the photon array.
      xmin = xin(0)
      xmax = xin(nen)
      
         
CC  define new x grid which is log spaced and matched to the data in a way that is exact for log bins 

      delta = log10(xmax/xmin)/(nen)
      deltal=delta*log(10.)

CC      delta=0.02
CC      deltal=delta*log(10.)
CC      xmin=1e-4*tempbb
CC      xmax=40*theta
CC      jmax=min(nen-1,int(log10(xmax/xmin)/delta)+1) 
C

CC     JS edits
CC      xmin = x(0)*0.01
CC      xmax = 40*theta


      do 37 j=0,nen
         x(j) = xmin*10.**(j*delta)
 37     continue

      
C X - ARRAY FOR PHOTON ENERGIES
C
CC      do 4 j=1,jmax+1
CC         x(j)=xmin*10.**((j-1)*delta)
CC  4   continue
c
c compute c2(x), and rel(x) arrays
c c2(x) is the relativistic correction to Kompaneets equation
c rel(x) is the Klein-Nishina cross section divided by the Thomson crossection




      do 500 j=1,nen-1
        w=x(j)
c c2 is the Cooper's coefficient calculated at w1
c w1 is x(j+1/2) (x(i) defined up to jmax+1)
        w1=sqrt(x(j)*x(j+1))
        c2(j)=w1**4/(1+4.6*w1+1.1*w1*w1)
        if (w.le.0.05) then
c use asymptotic limit for rel(x) for x less than 0.05
          rel(j)=1-2*w+26*w*w/5
        else
          z1=(1+w)/w**3
          z2=1+2*w
          z3=log(z2)
          z4=2*w*(1+w)/z2
          z5=z3/2/w
          z6=(1+3*w)/z2/z2
          rel(j)=0.75*(z1*(z4-z3)+z5-z6)
        end if
 500  continue


c the thermal emision spectrum 
CC        jmaxth=min(nen,int(log10(50*tempbb/xmin)/delta))
CC        if (jmaxth .gt. jmax) then
c           print *,'thcomp: ',jmaxth,jmax
CC           jmaxth = jmax
CC        end if
CC        planck=15/(pi*tempbb)**4
CC        do 5 j=1,jmaxth
CC          dphdot(j)=planck*x(j)**2/(exp(x(j)/tempbb)-1)
CC  5     continue

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCcc
CC JS --      in case of exact log binning matches, then can simply take it as is

      dif1 = 0.
      do 523 i=0,nen 
         dif1 = dif1+abs(xin(i)-x(i))
 523  continue


      IFLAG = .TRUE.            ! INTERP NEEDED IF TRUE
      if (dif1 .lt. 1e-3) then 
         dphdot = specin
         IFLAG = .FALSE.   ! SAME GRID so NO INTERP
      else                      ! interpolate to get a new array
         j=1
         do i=1,nen
            do while (j .le. (nen-1) .and. xin(j) .lt. x(i))
               j=j+1
            end do
            if (j .lt. nen) then
               if (j .gt. 1) then
                  jl = j-1
                  dphdot(i) = specin(jl)+(x(i)-xin(jl))
     >                 *(specin(jl+1)-specin(jl))/(xin(jl+1)-xin(jl))
               else
                  dphdot(i)=specin(1)
               endif
            else
               dphdot(i)=specin(j)
            endif
         end do
      endif


      dphdot(nen)=specin(nen)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCccc

      
c
c compute beta array, the probability of escape per Thomson time.
c bet evaluated for spherical geometry and nearly uniform sources.
c Between x=0.1 and 1.0, a function flz modifies beta to allow
c the increasingly large energy change per scattering to gradually
c eliminate spatial diffusion
      tmp1=minloc(abs(0.1-x))
      jnr = tmp1(1)-1

      tmp1 = minloc(abs(1.-x))
      jrel= tmp1(1)-1

      xnr=x(jnr)
      xr=x(jrel)
      do  531 j=1, jnr-1
         taukn=tautom*rel(j)
         bet(j)= 1/tautom/(1+taukn/3)
 531  continue
      do 600 j=jnr,jrel
         taukn=tautom*rel(j)
         arg=(x(j)-xnr)/(xr-xnr)
         flz=1-arg
         bet(j)=1/tautom/(1+taukn/3*flz)
 600   continue
       do 601 j=jrel+1,nen
          bet(j)=1/tautom
 601   continue
c
       call sthermlc 
     >      (tautom,theta,deltal,x,nen,dphesc,dphdot,bet,c2)
c
c     the spectrum in E F_E

       sptot(nen)=0.      
       do 497 j=1,nen-1
          sptot(j)=dphesc(j)*x(j)**2
 497   continue

c     the input spectrum


CC       open(21,file='scutx.out')
CC       do 499 j=1,nen
CC          write(21,*) x(j)*511., dphdot(j)*x(j)**2, xin(j)*511,specin(j)
CC     >      ,dphesc(j),sptot(j),theta*511.,gamma,dif1,IFLAG
CC 499   continue
CC       close(21)
      

      return  
      end


c ------------------------------------------------------------------ c
 
      REAL FUNCTION SPP(Y,Xnonth,Nnonth,Spnth)
 
      IMPLICIT NONE
      INTEGER Nnonth
      REAL Y , Xnonth(Nnonth) , Spnth(Nnonth)
      INTEGER il , ih
      REAL xx
CC      SAVE ih
CC      DATA ih/2/
 
      xx = 1/Y
      IF ( xx.LT.Xnonth(ih) ) ih = 2
 100  IF ( xx.GT.Xnonth(ih) ) THEN
         ih = ih + 1
         IF ( ih.LT.Nnonth ) GOTO 100
      ENDIF
      il = ih - 1
      SPP = Spnth(il) + (Spnth(ih)-Spnth(il))*(xx-Xnonth(il))
     &      /(Xnonth(ih)-Xnonth(il))
 
      RETURN
      END


      subroutine sthermlc 
     >  (tautom,theta,deltal,x,nen,dphesc,dphdot,bet,c2)
c This program computes the effects of Comptonization by
c nonrelativistic thermal electrons in a sphere including escape, and
c relativistic corrections up to photon energies of 1 MeV.
c the dimensionless photon energy is x=hv/(m*c*c)
c
c The input parameters and functions are:
c dphdot(x), the photon production rate 
c tautom, the Thomson scattering depth
c theta, the temperature in units of m*c*c
c c2(x), and bet(x), the coefficients in the K-equation and the
c   probability of photon escape per Thomson time, respectively,
c   including Klein-Nishina corrections
c The output parameters and functions are:
c dphesc(x), the escaping photon density
      implicit none 
      integer  nen
      real tautom,theta
      real deltal
      real x(0:nen),dphesc(nen), dphdot(nen),bet(nen),c2(nen)
      integer j,jj
      real a(nen),b(nen),c(nen),c20,w1,w2,t1,t2,t3,x32,aa
      real d(nen),alp(nen),g(nen),gam(nen),u(nen)
c u(x) is the dimensionless photon occupation number

      c20=tautom/deltal  

c     
c determine u
c define coefficients going into equation
c a(j)*u(j+1)+b(j)*u(j)+c(j)*u(j-1)=d(j)
      do 1 j=2,nen-1
         w1=sqrt(x(j)*x(j+1))
         w2=sqrt(x(j-1)*x(j))
c     w1 is x(j+1/2)
c     w2 is x(j-1/2)
         a(j)=-c20*c2(j)*(theta/deltal/w1+0.5)
         t1=-c20*c2(j)*(0.5-theta/deltal/w1)
         t2=c20*c2(j-1)*(theta/deltal/w2+0.5)
         t3=x(j)**3*(tautom*bet(j))
         b(j)=t1+t2+t3
         c(j)=c20*c2(j-1)*(0.5-theta/deltal/w2)
         d(j)=x(j)*dphdot(j)
 1    continue

c define constants going into boundary terms
c u(1)=aa*u(2) (zero flux at lowest energy)
c u(jx2) given from region 2 above
      x32=sqrt(x(1)*x(2))
      aa=(theta/deltal/x32+0.5)/(theta/deltal/x32-0.5)
c
c zero flux at the highest energy
      u(nen)=0
c
c invert tridiagonal matrix
      alp(2)=b(2)+c(2)*aa
      gam(2)=a(2)/alp(2)
      do 2 j=3,nen-1
        alp(j)=b(j)-c(j)*gam(j-1)
        gam(j)=a(j)/alp(j)
  2   continue
      g(2)=d(2)/alp(2)
      do 3 j=3,nen-2
        g(j)=(d(j)-c(j)*g(j-1))/alp(j)
  3   continue
      g(nen-1)=
     > (d(nen-1)-a(nen-1)*u(nen)-c(nen-1)*g(nen-2))/alp(nen-1)
      u(nen-1)=g(nen-1)
      do 4 j=3,nen-1
        jj=nen+1-j
        u(jj)=g(jj)-gam(jj)*u(jj+1)
  4   continue
      u(1)=aa*u(2)

c compute new value of dph(x) and new value of dphesc(x)
      do 9 j=1,nen-1
        dphesc(j)=x(j)*x(j)*u(j)*bet(j)*tautom
 9    continue
      
C      open(9,file='Z.out')
C      do 99 j=1,nen-1
C         write(9,*) x(j)*511,x32,aa,u(j),g(j),d(j),c(j),bet(j),dphesc(j)
C 99   continue
C      close(9)
     
      return
      end
